# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl FaiTS.t'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

#use Test::More tests => 1;
use strict;
use warnings;

use Test::More 'no_plan';
use File::Path;
use File::Temp qw(tempdir);
use Hash::Merge qw(merge);
use YAML qw(LoadFile);

#############################################################################
# Create Test-Enviroment first, tests are at end
#############################################################################

#Set Variables:
my %ignoreDirs = map { $_, 1 } qw/CVS .svn .arch-ids {arch}/;
my @classes = qw(DEFAULT LINUX I386 DEMO ROUTER demohost LAST);

# Create directories to run tests in
my $faiDir = tempdir('tmpXXXXX', DIR => '.', CLEANUP => 1);
my $prefixTemplateDir = "$faiDir/templates";
my $prefixTargetDir   = "$faiDir/target";
my $prefixClassDir    = "$faiDir/class";
mkpath([$prefixTargetDir, $prefixClassDir, "$prefixTemplateDir/etc/network/interfaces"]);

# Create YAML config files
open ROUTERYML, ">$prefixClassDir/ROUTER.yml" or die $!;
print ROUTERYML <<'EOF';
interfaces:
  eth0:
    address: 172.16.240.5
    netmask: 255.255.255.0
    network: 172.16.240.0
    broadcast: 172.16.240.255
  eth1:
    address: 192.168.1.5
    netmask: 255.255.255.0
    network: 192.168.1.0
    broadcast: 192.168.1.255
  eth2:
    address: 192.168.1.5
    netmask: 255.255.255.0
    network: 192.168.1.0
    broadcast: 192.168.1.255
    gateway: 212.202.236.177
EOF
close ROUTERYML;

open DEMOYML, ">$prefixClassDir/DEMO.yml" or die $!;
print DEMOYML <<'EOF';
interfaces:
  eth3:
    address: test
  eth0:
    address: bla
EOF
close DEMOYML;

#read yml-files and merge config data
my @ymlFiles = grep { -f "$prefixClassDir/$_.yml" } @classes;
@ymlFiles    = map { $_ = "$prefixClassDir/$_.yml" } @ymlFiles;
my $config = {};
$config = merge( $config, LoadFile($_) || die $! ) foreach @ymlFiles;

# Create template files
open ROUTERTT, ">$prefixTemplateDir/etc/network/interfaces/ROUTER.tt" or die $!;
print ROUTERTT <<'EOF';
#ROUTER
# /etc/network/interfaces -- configuration file for ifup(8), ifdown(8)

# The loopback interface
auto lo
iface lo inet loopback

[% FOREACH name = interfaces.keys.sort %]
[% i = interfaces.$name -%]
auto [% name %]
iface [% name %] inet static
        address   [% i.address %]
        netmask   [% i.netmask %]
       [% IF i.network   %] network   [% i.network   %] [% END %]
       [% IF i.broadcast %] broadcast [% i.broadcast %] [% END %]
       [% IF i.gateway   %] gateway   [% i.gateway   %] [% END %]

[% END %]
EOF
close ROUTERTT;

open DEMOTT, ">$prefixTemplateDir/etc/network/interfaces/DEMO.tt" or die $!;
print DEMOTT <<'EOF';
#DEMO
# The loopback interface
auto lo
iface lo inet loopback

auto [% interfaces.eth0.name %]
iface [% interfaces.eth0.name %] inet static
        address   [% i.address %]
        netmask   [% i.netmask %]

EOF
close DEMOTT;

#############################################################################
# Tests start here:
#############################################################################

BEGIN { 
  use_ok('FaiTS', qw(dirIsWorldWritable findTemplates writeTargetFiles)) 
};

SKIP: {
skip 'not ready for production ...';
ok( ! dirIsWorldWritable("$prefixTemplateDir/etc/network/interfaces"), 'existing directory' );
ok( ! dirIsWorldWritable("$prefixTemplateDir/etc/network/gibtsnicht"), 'non-existing directory' );
}
ok( ( my $templates = findTemplates(\%ignoreDirs, \@classes, $prefixTemplateDir)), 'templates variables created' );
ok( $templates->{'etc/network/interfaces'}->{ROUTER}, 'template found' );
ok( writeTargetFiles($templates, \@classes, $config, $prefixTemplateDir, $prefixTargetDir) , 'writeTargetFiles');
ok( -f "$prefixTargetDir/etc/network/interfaces", 'target file exists');

